% Written by ngelo Cardoso <angelo.cardoso@ist.utl.pt>
% 27 December 2012

% The detail of the algorithm is described in the following paper:
% ngelo Cardoso, Andreas Wichert, Iterative random projections for
% high-dimensional data clustering, Pattern Recognition Letters, Volume 33,
% Issue 13, 1 October 2012, Pages 1749-1755, ISSN 0167-8655

function [G, C, Diter] = irpkmeans(X,k,D, maxIter, KmeansFun,singleton)
%IRP K-means algorithm

%input
% X -data (samples x dimensions)

% k - number of clusters

% D (optional) - list of dimensions (default: set using dheur)
% if length(D)=1 is equivalent to RP K-means algorithm

% maxIter (optional) - maximum iterations for K-means (default: inf)

% KmeansFun (optional) - K-means implementation (default: @simplekmeans)

% singleton (optional) -  replace emtpy clusters by furthest point 
% from centroids (default: false)

%output
% G - cluster membership

% Diter - number of iterations in dimension D_a

%Parse arguments
error(nargchk(2,6, nargin));
if nargin < 6
   singleton = []; 
end
if nargin < 5
    KmeansFun = [];
end
if nargin < 4
    maxIter = [];
end
if nargin < 3
    D = [];
end
if(isempty(D))
    %if D is not set find D according to heuristic
    display('finding D...');
    [D] = dheur(X,k,[],[],[]);
    [D] = sort(D,'ascend');
    display(['D=' num2str(D)]);
end
if isempty(maxIter)
    %default number of iterations
    maxIter = inf;
end
if isempty(KmeansFun)
    %default K-means implementation
    KmeansFun = @simplekmeans;
end
if isempty(singleton)
    %default action for empty clusters
    singleton = false;
end


%Set the cluster membership G as K randomly sampled points from X
G = nan(size(X,1),1);
G(randsamplewr(size(X,1),k))=1:k;

for i=1:length(D)
    %Set a random matrix P
    P = randn(size(X,2),D(i));
    
    %Set X_RP by projecting X into P
    X_RP = X*P;
    
    
    %Set C_RP by calculating the mean of each cluster in X_RP according to G
    C_RP =  calccenters(X_RP,G);
    
    %Obtain G with K-means with C_RP as initialization
    [G, Diter(i)] = KmeansFun(X_RP,C_RP,maxIter,singleton);
end

if nargout > 1
    C = calccenters(X,G);
end

end

